==> hubzero-mw-expire_0.8.2.orig.tar.gz: hubzero-mw-expire-0.8.2/expire-sessions.py <==
#!/usr/bin/env python

import os,pwd,sys,time,ldap,MySQLdb,signal

# always run in /var/lib/mw
os.chdir("/var/lib/mw")

foreground = 0

expire_interval = 15
logfile_name = "/var/log/mw/expire.log"
logfile = None
pidfile = "/var/run/expire-sessions.pid"
mwprog = "/usr/lib/mw/bin/maxwell"

#==============================================================================
# Default values.
#==============================================================================

mysql_host = ""
mysql_user=""
mysql_password=""
mysql_db=""
mysql_connect_attempts=120

#=============================================================================
# Load the configuration and override the variables above.
#=============================================================================
try:
  execfile('/etc/mw/maxwell.conf')
except IOError:
  pass

#=============================================================================
# Set up errors to go to the log file.
#=============================================================================
def openlog(filename):
  global logfile
  global foreground

  if not foreground:
    try:
      logfile = open(filename,"a+")
      devnull = open("/dev/null", "rw")
      os.close(sys.stdin.fileno())
      os.close(sys.stdout.fileno())
      os.close(sys.stderr.fileno())
      os.dup2(devnull.fileno(), 0)
      os.dup2(logfile.fileno(), 1)
      os.dup2(logfile.fileno(), 2)
      os.dup2(sys.stdin.fileno(), devnull.fileno())
      return
    except:
      pass

#=============================================================================
# Log a message.
#=============================================================================
def log(msg):
  if not foreground:
    timestamp = "[" + time.asctime() + "] "
  else:
    timestamp = ""
  logfile.write(timestamp + msg + "\n")
  logfile.flush()

#=============================================================================
# Create database connection.
#=============================================================================
def db_connect():
  for x in range(0,mysql_connect_attempts):
    try:
      db = MySQLdb.connect(host=mysql_host, user=mysql_user, passwd=mysql_password, db=mysql_db)
      #log("db_connect finished on iteration %d" % x)
      return db
    except:
      log("Exception in db_connect")
    time.sleep(1)

#=============================================================================
# MySQL helpers
#=============================================================================
def mysql(c,cmd):
  try:
    count = c.execute(cmd)
    if c._warnings > 0:
      log("MySQL warning")
      log("SQL was: %s" % cmd)
    return c.fetchall()
  except MySQLdb.MySQLError, (num, expl):
    log("%s" % expl)
    log("SQL was: %s" % cmd)
    return ()
  except:
    log("Some other MySQL exception.")
    log("SQL was: %s" % cmd)
    return ()

def mysql_act(c,cmd):
  try:
    count = c.execute(cmd)
    return ""
  except MySQLdb.MySQLError, (num, expl):
    return expl

#=============================================================================
# Daemonize.
#=============================================================================
def daemonize():
  #log("Backgrounding process.")
  if os.fork():
    os._exit(0)
  os.setsid()
  os.chdir("/")
  if os.fork():
    os._exit(0)
  try:
    f = open(pidfile,"w")
    f.write("%d\n" % os.getpid())
    f.close()
  except:
    log("Unable to write pid (%d) to %s" % (os.getpid(),pidfile))

#=============================================================================
# Change to different user.
#=============================================================================
def change_user(username):
  if os.getuid() == 0:
    try:
      _,_,uid,gid,_,_,_ = pwd.getpwnam("www-data")
      os.setregid(gid,gid)
      os.setreuid(uid,uid)
    except:
      pass

#=============================================================================
# Handle signals.
#=============================================================================
def sighandler(sig,frame):
  log("Caught signal %d.  Exiting." % sig)
  sys.exit(1)

#=============================================================================
# Do the work of terminating a session.
#=============================================================================
def terminate(sessnum,username,appname):
  log("Terminating %d (%s,%s)" % (sessnum,username,appname))
  status=os.system("%s stop reason=timeout sessnum=%d" % (mwprog,sessnum))
  if status != 0:
    log("Error terminating session: %d" % status)

#=============================================================================
# Main program
#=============================================================================

signal.signal(signal.SIGHUP, sighandler)
signal.signal(signal.SIGINT, sighandler)
signal.signal(signal.SIGQUIT, sighandler)
signal.signal(signal.SIGTERM, sighandler)

if not foreground:
  daemonize()
  change_user("www-data")
  openlog(logfile_name)
  log("Server is ready.")

while 1:
  db = db_connect()
  c = db.cursor()
  arr = mysql(c, """
           SELECT session.sessnum,session.username,appname,
           timeout-TIME_TO_SEC(TIMEDIFF(NOW(), accesstime)) AS remaining
           FROM session
           LEFT JOIN view ON session.sessnum = view.sessnum
           LEFT JOIN job ON session.sessnum = job.sessnum
           WHERE viewid IS NULL AND jobid IS NULL AND
           timeout-TIME_TO_SEC(TIMEDIFF(NOW(), accesstime)) < %d
           ORDER BY REMAINING""" % expire_interval)
  db.close()

  # Recheck, at the longest, every "expire_interval" seconds.
  shortest = expire_interval
  skew=0
  for sessnum,username,appname,remaining in arr:
    if foreground:
      log("%d of %s to expire in %d seconds" % (sessnum,username,remaining))
    if remaining < 1 + skew:
      terminate(sessnum,username,appname)
      time.sleep(1) # Do not kill things off too quickly.
      skew += 1
    elif remaining < shortest:
      shortest = remaining

  shortest -= skew
  if (shortest > 0):
    time.sleep(shortest)

==> hubzero-mw-expire_0.9.0.orig.tar.gz: hubzero-mw-expire-0.9.0/expire-sessions.py <==
#!/usr/bin/env python

import os,pwd,sys,time,ldap,MySQLdb,signal

# always run in /var/lib/hubzero
os.chdir("/var/lib/hubzero")

foreground = 0

expire_interval = 15
logfile_name = "/var/log/hubzero/expire.log"
logfile = None
pidfile = "/var/run/expire-sessions.pid"
mwprog = "/usr/lib/hubzero/bin/maxwell"

#==============================================================================
# Default values.
#==============================================================================

mysql_host = ""
mysql_user=""
mysql_password=""
mysql_db=""
mysql_connect_attempts=120

#=============================================================================
# Load the configuration and override the variables above.
#=============================================================================
try:
  execfile('/etc/hubzero/maxwell.conf')
except IOError:
  pass

#=============================================================================
# Set up errors to go to the log file.
#=============================================================================
def openlog(filename):
  global logfile
  global foreground

  if not foreground:
    try:
      logfile = open(filename,"a+")
      devnull = open("/dev/null", "rw")
      os.close(sys.stdin.fileno())
      os.close(sys.stdout.fileno())
      os.close(sys.stderr.fileno())
      os.dup2(devnull.fileno(), 0)
      os.dup2(logfile.fileno(), 1)
      os.dup2(logfile.fileno(), 2)
      os.dup2(sys.stdin.fileno(), devnull.fileno())
      return
    except:
      pass

#=============================================================================
# Log a message.
#=============================================================================
def log(msg):
  if not foreground:
    timestamp = "[" + time.asctime() + "] "
  else:
    timestamp = ""
  logfile.write(timestamp + msg + "\n")
  logfile.flush()

#=============================================================================
# Create database connection.
#=============================================================================
def db_connect():
  for x in range(0,mysql_connect_attempts):
    try:
      db = MySQLdb.connect(host=mysql_host, user=mysql_user, passwd=mysql_password, db=mysql_db)
      #log("db_connect finished on iteration %d" % x)
      return db
    except:
      log("Exception in db_connect")
    time.sleep(1)

#=============================================================================
# MySQL helpers
#=============================================================================
def mysql(c,cmd):
  try:
    count = c.execute(cmd)
    if c._warnings > 0:
      log("MySQL warning")
      log("SQL was: %s" % cmd)
    return c.fetchall()
  except MySQLdb.MySQLError, (num, expl):
    log("%s" % expl)
    log("SQL was: %s" % cmd)
    return ()
  except:
    log("Some other MySQL exception.")
    log("SQL was: %s" % cmd)
    return ()

def mysql_act(c,cmd):
  try:
    count = c.execute(cmd)
    return ""
  except MySQLdb.MySQLError, (num, expl):
    return expl

#=============================================================================
# Daemonize.
#=============================================================================
def daemonize():
  #log("Backgrounding process.")
  if os.fork():
    os._exit(0)
  os.setsid()
  os.chdir("/")
  if os.fork():
    os._exit(0)
  try:
    f = open(pidfile,"w")
    f.write("%d\n" % os.getpid())
    f.close()
  except:
    log("Unable to write pid (%d) to %s" % (os.getpid(),pidfile))

#=============================================================================
# Change to different user.
#=============================================================================
def change_user(username):
  if os.getuid() == 0:
    try:
      _,_,uid,gid,_,_,_ = pwd.getpwnam("www-data")
      os.setregid(gid,gid)
      os.setreuid(uid,uid)
    except:
      pass

#=============================================================================
# Handle signals.
#=============================================================================
def sighandler(sig,frame):
  log("Caught signal %d.  Exiting." % sig)
  sys.exit(1)

#=============================================================================
# Do the work of terminating a session.
#=============================================================================
def terminate(sessnum,username,appname):
  log("Terminating %d (%s,%s)" % (sessnum,username,appname))
  status=os.system("%s stop reason=timeout sessnum=%d" % (mwprog,sessnum))
  if status != 0:
    log("Error terminating session: %d" % status)

#=============================================================================
# Main program
#=============================================================================

signal.signal(signal.SIGHUP, sighandler)
signal.signal(signal.SIGINT, sighandler)
signal.signal(signal.SIGQUIT, sighandler)
signal.signal(signal.SIGTERM, sighandler)

if not foreground:
  daemonize()
  change_user("www-data")
  openlog(logfile_name)
  log("Server is ready.")

while 1:
  db = db_connect()
  c = db.cursor()
  arr = mysql(c, """
           SELECT session.sessnum,session.username,appname,
           timeout-TIME_TO_SEC(TIMEDIFF(NOW(), accesstime)) AS remaining
           FROM session
           LEFT JOIN view ON session.sessnum = view.sessnum
           LEFT JOIN job ON session.sessnum = job.sessnum
           WHERE viewid IS NULL AND jobid IS NULL AND
           timeout-TIME_TO_SEC(TIMEDIFF(NOW(), accesstime)) < %d
           ORDER BY REMAINING""" % expire_interval)
  db.close()

  # Recheck, at the longest, every "expire_interval" seconds.
  shortest = expire_interval
  skew=0
  for sessnum,username,appname,remaining in arr:
    if foreground:
      log("%d of %s to expire in %d seconds" % (sessnum,username,remaining))
    if remaining < 1 + skew:
      terminate(sessnum,username,appname)
      time.sleep(1) # Do not kill things off too quickly.
      skew += 1
    elif remaining < shortest:
      shortest = remaining

  shortest -= skew
  if (shortest > 0):
    time.sleep(shortest)
